@@ -49,14 +49,21 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.time.Duration;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
+import java.util.SortedMap;
+import java.util.TreeMap;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.Consumer;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import static org.apache.flink.configuration.CheckpointingOptions.SAVEPOINT_DIRECTORY;
 import static org.apache.flink.kubernetes.operator.config.FlinkConfigBuilder.applyJobConfig;
@@ -80,6 +87,14 @@ public class FlinkConfigManager {
     private final AtomicLong defaultConfigVersion = new AtomicLong(0);
     private final LoadingCache<Key, Configuration> cache;
     private final Consumer<Set<String>> namespaceListener;
+    private volatile ConcurrentHashMap<FlinkVersion, List<String>> relevantFlinkVersionPrefixes;
+
+    protected static final Pattern FLINK_VERSION_PATTERN =
+            Pattern.compile(
+                    VERSION_CONF_PREFIX.replaceAll("\\.", "\\\\\\.")
+                            + "v(?<major>\\d+)_(?<minor>\\d+)(?<gt>\\"
+                            + KubernetesOperatorConfigOptions.FLINK_VERSION_GREATER_THAN_SUFFIX
+                            + ")?\\..*");
 
     @VisibleForTesting
     public FlinkConfigManager(Configuration defaultConfig) {
@@ -99,6 +114,7 @@ public class FlinkConfigManager {
         this.namespaceListener = namespaceListener;
         Duration cacheTimeout =
                 defaultConfig.get(KubernetesOperatorConfigOptions.OPERATOR_CONFIG_CACHE_TIMEOUT);
+        this.relevantFlinkVersionPrefixes = new ConcurrentHashMap<>();
         this.cache =
                 CacheBuilder.newBuilder()
                         .maximumSize(
@@ -169,6 +185,11 @@ public class FlinkConfigManager {
         // We do not invalidate the cache to avoid deleting currently used temp files,
         // simply bump the version
         this.defaultConfigVersion.incrementAndGet();
+
+        // We clear the cached relevant Flink version prefixes as the base config may include new
+        // version overrides.
+        // This will trigger a regeneration of the prefixes in the next call to getDefaultConfig.
+        relevantFlinkVersionPrefixes = new ConcurrentHashMap<>();
     }
 
     /**
@@ -202,6 +223,65 @@ public class FlinkConfigManager {
                 getDefaultConfig(namespace, flinkVersion));
     }
 
+    /**
+     * This method will search the keys of the supplied map and find any that contain a flink
+     * version string that is relevant to the supplied flink version.
+     *
+     * <p>Relevance is defined as any key with the {@link
+     * KubernetesOperatorConfigOptions#VERSION_CONF_PREFIX} followed by either the supplied flink
+     * version (with or without the {@link
+     * KubernetesOperatorConfigOptions#FLINK_VERSION_GREATER_THAN_SUFFIX}) or a lower flink version
+     * string followed by the {@link
+     * KubernetesOperatorConfigOptions#FLINK_VERSION_GREATER_THAN_SUFFIX}.
+     *
+     * <p>Prefixes are returned in ascending order of flink version.
+     *
+     * @param baseConfMap The configuration map that should be searched for relevant Flink version
+     *     prefixes.
+     * @return A list of relevant Flink version prefixes in order of ascending Flink version.
+     */
+    protected static List<String> getRelevantVersionPrefixes(
+            Map<String, String> baseConfMap, FlinkVersion flinkVersion) {
+        SortedMap<FlinkVersion, String> greaterThanVersionPrefixes = new TreeMap<>();
+
+        for (Map.Entry<String, String> entry : baseConfMap.entrySet()) {
+            Matcher versionMatcher = FLINK_VERSION_PATTERN.matcher(entry.getKey());
+            if (versionMatcher.matches() && versionMatcher.group("gt") != null) {
+                try {
+                    FlinkVersion keyFlinkVersion =
+                            FlinkVersion.fromMajorMinor(
+                                    Integer.parseInt(versionMatcher.group("major")),
+                                    Integer.parseInt(versionMatcher.group("minor")));
+                    if (flinkVersion.isEqualOrNewer(keyFlinkVersion)) {
+                        greaterThanVersionPrefixes.put(
+                                keyFlinkVersion,
+                                VERSION_CONF_PREFIX
+                                        + keyFlinkVersion
+                                        + KubernetesOperatorConfigOptions
+                                                .FLINK_VERSION_GREATER_THAN_SUFFIX
+                                        + ".");
+                    }
+                } catch (NumberFormatException numberFormatException) {
+                    LOG.warn("Unable to parse version number in config key: {}", entry.getKey());
+                } catch (IllegalArgumentException illegalArgumentException) {
+                    LOG.warn("Unknown Flink version in config key: {}", entry.getKey());
+                }
+            }
+        }
+
+        // Extract the prefixes from the sorted map, these will be ascending Flink version order
+        List<String> sortedRelevantVersionPrefixes =
+                new ArrayList<>(greaterThanVersionPrefixes.values());
+
+        // Add the current flink version prefix (without the greater than symbol) to the set.
+        // Any current flink version prefix with the greater than symbol would already have been
+        // added
+        // in the loop above.
+        sortedRelevantVersionPrefixes.add(VERSION_CONF_PREFIX + flinkVersion + ".");
+
+        return sortedRelevantVersionPrefixes;
+    }
+
     /**
      * Get the base configuration for the given namespace and flink version combination. This is
      * different from the platform level base config as it may contain namespaces or version
@@ -220,7 +300,20 @@ public class FlinkConfigManager {
         }
 
         if (flinkVersion != null) {
-            applyDefault(VERSION_CONF_PREFIX + flinkVersion + ".", baseConfMap, conf);
+            // Fetch or create a list of Flink version configs that apply to this current
+            // FlinkVersion. That will include all versions that are equal to or lower than
+            // the current one that are suffixed by a `+`
+            List<String> versionPrefixes =
+                    relevantFlinkVersionPrefixes.computeIfAbsent(
+                            flinkVersion,
+                            fv -> getRelevantVersionPrefixes(baseConfMap, flinkVersion));
+
+            // The version prefixes are returned in ascending order of Flink version, so configs
+            // attached to newer versions will override older ones. For example v1_16+.conf1 will
+            // be overridden if a key containing v1_18+.conf1 is present.
+            for (String versionPrefix : versionPrefixes) {
+                applyDefault(versionPrefix, baseConfMap, conf);
+            }
         }
 
         return conf;
